@@ -13,12 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
 import novaclient
 from novaclient import api_versions
 from novaclient import exceptions
-from novaclient.openstack.common import cliutils
 from novaclient.tests.unit import utils
 from novaclient import utils as nutils
 from novaclient.v2 import versions
@@ -86,12 +85,12 @@ class APIVersionTestCase(utils.TestCase):
         v4 = api_versions.APIVersion("2.0")
         v_null = api_versions.APIVersion()
 
-        self.assertTrue(v1 < v2)
-        self.assertTrue(v3 > v2)
-        self.assertTrue(v1 != v2)
-        self.assertTrue(v1 == v4)
-        self.assertTrue(v1 != v_null)
-        self.assertTrue(v_null == v_null)
+        self.assertTrue(v1.__lt__(v2))
+        self.assertTrue(v3.__gt__(v2))
+        self.assertTrue(v1.__ne__(v2))
+        self.assertTrue(v1.__eq__(v4))
+        self.assertTrue(v1.__ne__(v_null))
+        self.assertTrue(v_null.__eq__(v_null))
         self.assertRaises(TypeError, v1.__le__, "2.1")
 
     def test_version_matches(self):
@@ -145,6 +144,18 @@ class UpdateHeadersTestCase(utils.TestCase):
             {"X-OpenStack-Nova-API-Version": api_version.get_string()},
             headers)
 
+    def test_api_version_is_gte_27(self):
+        api_version = api_versions.APIVersion("2.27")
+        headers = {}
+        api_versions.update_headers(headers, api_version)
+        self.assertIn('X-OpenStack-Nova-API-Version', headers)
+        self.assertIn('OpenStack-API-Version', headers)
+        self.assertEqual(api_version.get_string(),
+                         headers['X-OpenStack-Nova-API-Version'])
+        self.assertEqual('%s %s' % (api_versions.SERVICE_TYPE,
+                                    api_version.get_string()),
+                         headers['OpenStack-API-Version'])
+
 
 class CheckHeadersTestCase(utils.TestCase):
     def setUp(self):
@@ -153,8 +164,9 @@ class CheckHeadersTestCase(utils.TestCase):
         self.mock_log = mock_log_patch.start()
         self.addCleanup(mock_log_patch.stop)
 
-    def test_microversion_is_specified(self):
-        response = mock.MagicMock(headers={api_versions.HEADER_NAME: ""})
+    def test_legacy_microversion_is_specified(self):
+        response = mock.MagicMock(
+            headers={api_versions.LEGACY_HEADER_NAME: ""})
         api_versions.check_headers(response, api_versions.APIVersion("2.2"))
         self.assertFalse(self.mock_log.warning.called)
 
@@ -162,8 +174,19 @@ class CheckHeadersTestCase(utils.TestCase):
         api_versions.check_headers(response, api_versions.APIVersion("2.2"))
         self.assertTrue(self.mock_log.warning.called)
 
+    def test_generic_microversion_is_specified(self):
+        response = mock.MagicMock(
+            headers={api_versions.HEADER_NAME: ""})
+        api_versions.check_headers(response, api_versions.APIVersion("2.27"))
+        self.assertFalse(self.mock_log.warning.called)
+
+        response = mock.MagicMock(headers={})
+        api_versions.check_headers(response, api_versions.APIVersion("2.27"))
+        self.assertTrue(self.mock_log.warning.called)
+
     def test_microversion_is_not_specified(self):
-        response = mock.MagicMock(headers={api_versions.HEADER_NAME: ""})
+        response = mock.MagicMock(
+            headers={api_versions.LEGACY_HEADER_NAME: ""})
         api_versions.check_headers(response, api_versions.APIVersion("2.2"))
         self.assertFalse(self.mock_log.warning.called)
 
@@ -210,11 +233,9 @@ class WrapsTestCase(utils.TestCase):
         m.name = args[0]
         return m
 
-    @mock.patch("novaclient.utils.get_function_name")
+    @mock.patch("novaclient.api_versions._get_function_name")
     @mock.patch("novaclient.api_versions.VersionedMethod")
     def test_end_version_is_none(self, mock_versioned_method, mock_name):
-        func_name = "foo"
-        mock_name.return_value = func_name
         mock_versioned_method.side_effect = self._side_effect_of_vers_method
 
         @api_versions.wraps("2.2")
@@ -224,15 +245,13 @@ class WrapsTestCase(utils.TestCase):
         foo(self._get_obj_with_vers("2.4"))
 
         mock_versioned_method.assert_called_once_with(
-            func_name, api_versions.APIVersion("2.2"),
+            mock_name.return_value, api_versions.APIVersion("2.2"),
             api_versions.APIVersion("2.latest"), mock.ANY)
 
-    @mock.patch("novaclient.utils.get_function_name")
+    @mock.patch("novaclient.api_versions._get_function_name")
     @mock.patch("novaclient.api_versions.VersionedMethod")
     def test_start_and_end_version_are_presented(self, mock_versioned_method,
                                                  mock_name):
-        func_name = "foo"
-        mock_name.return_value = func_name
         mock_versioned_method.side_effect = self._side_effect_of_vers_method
 
         @api_versions.wraps("2.2", "2.6")
@@ -242,14 +261,12 @@ class WrapsTestCase(utils.TestCase):
         foo(self._get_obj_with_vers("2.4"))
 
         mock_versioned_method.assert_called_once_with(
-            func_name, api_versions.APIVersion("2.2"),
+            mock_name.return_value, api_versions.APIVersion("2.2"),
             api_versions.APIVersion("2.6"), mock.ANY)
 
-    @mock.patch("novaclient.utils.get_function_name")
+    @mock.patch("novaclient.api_versions._get_function_name")
     @mock.patch("novaclient.api_versions.VersionedMethod")
     def test_api_version_doesnt_match(self, mock_versioned_method, mock_name):
-        func_name = "foo"
-        mock_name.return_value = func_name
         mock_versioned_method.side_effect = self._side_effect_of_vers_method
 
         @api_versions.wraps("2.2", "2.6")
@@ -260,7 +277,7 @@ class WrapsTestCase(utils.TestCase):
                           foo, self._get_obj_with_vers("2.1"))
 
         mock_versioned_method.assert_called_once_with(
-            func_name, api_versions.APIVersion("2.2"),
+            mock_name.return_value, api_versions.APIVersion("2.2"),
             api_versions.APIVersion("2.6"), mock.ANY)
 
     def test_define_method_is_actually_called(self):
@@ -278,22 +295,53 @@ class WrapsTestCase(utils.TestCase):
 
         checker.assert_called_once_with(*((obj,) + some_args), **some_kwargs)
 
-    def test_arguments_property_is_copied(self):
-        @cliutils.arg("argument_1")
+    @mock.patch("novaclient.api_versions._get_function_name")
+    def test_arguments_property_is_copied(self, mock_name):
+        @nutils.arg("argument_1")
         @api_versions.wraps("2.666", "2.777")
-        @cliutils.arg("argument_2")
+        @nutils.arg("argument_2")
         def some_func():
             pass
 
         versioned_method = api_versions.get_substitutions(
-            nutils.get_function_name(some_func),
-            api_versions.APIVersion("2.700"))[0]
+            mock_name.return_value, api_versions.APIVersion("2.700"))[0]
 
         self.assertEqual(some_func.arguments,
                          versioned_method.func.arguments)
         self.assertIn((("argument_1",), {}), versioned_method.func.arguments)
         self.assertIn((("argument_2",), {}), versioned_method.func.arguments)
 
+    def test_several_methods_with_same_name_in_one_module(self):
+
+        class A(object):
+            api_version = api_versions.APIVersion("777.777")
+
+            @api_versions.wraps("777.777")
+            def f(self):
+                return 1
+
+        class B(object):
+            api_version = api_versions.APIVersion("777.777")
+
+            @api_versions.wraps("777.777")
+            def f(self):
+                return 2
+
+        self.assertEqual(1, A().f())
+        self.assertEqual(2, B().f())
+
+    def test_generate_function_name(self):
+        expected_name = "novaclient.tests.unit.test_api_versions.fake_func"
+
+        self.assertNotIn(expected_name, api_versions._SUBSTITUTIONS)
+
+        @api_versions.wraps("7777777.7777777")
+        def fake_func():
+            pass
+
+        self.assertIn(expected_name, api_versions._SUBSTITUTIONS)
+        self.assertEqual(expected_name, fake_func.__id__)
+
 
 class DiscoverVersionTestCase(utils.TestCase):
     def setUp(self):
@@ -390,3 +438,20 @@ class DiscoverVersionTestCase(utils.TestCase):
             api_versions.discover_version(
                 fake_client,
                 api_versions.APIVersion('2.latest')).get_string())
+
+
+class DecoratedAfterTestCase(utils.TestCase):
+    def test_decorated_after(self):
+
+        class Fake(object):
+            api_version = api_versions.APIVersion('2.123')
+
+            @api_versions.deprecated_after('2.123')
+            def foo(self):
+                pass
+
+        with mock.patch('warnings.warn') as mock_warn:
+            Fake().foo()
+            msg = ('The novaclient.tests.unit.test_api_versions module '
+                   'is deprecated and will be removed.')
+            mock_warn.assert_called_once_with(msg, mock.ANY)
