@@ -16,11 +16,11 @@ import distutils.version as dist_version
 import io
 import re
 import sys
+from unittest import mock
 
 import ddt
 import fixtures
 from keystoneauth1 import fixture
-import mock
 import prettytable
 import requests_mock
 from testtools import matchers
@@ -451,6 +451,9 @@ class ShellTest(utils.TestCase):
     def test_help_no_options(self):
         self._test_help('')
 
+    def test_help_no_subcommand(self):
+        self._test_help('--os-compute-api-version 2.87')
+
     def test_help_on_subcommand(self):
         required = [
             '.*?^usage: nova set-password',
